@@ -1,4 +1,4 @@
-/* $Id: recvfrom.h,v 1.2 2001/08/03 04:03:40 hpa Exp $ */
+/* $Id: recvfrom.h,v 1.3 2001/11/14 02:54:25 hpa Exp $ */
 /* ----------------------------------------------------------------------- *
  *   
  *   Copyright 2001 H. Peter Anvin - All Rights Reserved
@@ -16,12 +16,7 @@
  *
  */
 
-#define _XPG4_2			/* Needed on Solaris */
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
-#include <stdlib.h>
+#include "config.h"
 
 int
 myrecvfrom(int s, void *buf, int len, unsigned int flags,
